#!/bin/bash

{% if atomic %}
{{ sched_args }} python "$EXEC_PATH"/jobs.py full --src "${qs_src}" --cfg "${qs_cfg}"
{% else %}
{{ sched_args }} python "$EXEC_PATH"/jobs.py pre --src "${qs_src}" --cfg "${qs_cfg}"
{{ sched_args }} python "$EXEC_PATH"/jobs.py run --src "${qs_src}" --cfg "${qs_cfg}"
{{ sched_args }} python "$EXEC_PATH"/jobs.py post --src "${qs_src}" --cfg "${qs_cfg}"
{% endif %} 
