# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qpu.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\tqpu.proto\x12\x16qstone.connectors.grpc"8\n\x07\x43ircuit\x12\x0f\n\x07\x63ircuit\x18\x01 \x01(\t\x12\x0e\n\x06pkt_id\x18\x02 \x01(\x05\x12\x0c\n\x04reps\x18\x03 \x01(\x05"3\n\x0f\x43ircuitResponse\x12\x0e\n\x06result\x18\x01 \x01(\t\x12\x10\n\x08\x63\x61pacity\x18\x02 \x01(\x05\x32\x66\n\x03QPU\x12_\n\x11RunQuantumCircuit\x12\x1f.qstone.connectors.grpc.Circuit\x1a\'.qstone.connectors.grpc.CircuitResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "qpu_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_CIRCUIT"]._serialized_start = 37
    _globals["_CIRCUIT"]._serialized_end = 93
    _globals["_CIRCUITRESPONSE"]._serialized_start = 95
    _globals["_CIRCUITRESPONSE"]._serialized_end = 146
    _globals["_QPU"]._serialized_start = 148
    _globals["_QPU"]._serialized_end = 250
# @@protoc_insertion_point(module_scope)
